@@ -22,8 +22,10 @@ import defusedxml
 import numpy as np
 from PIL import Image
 
-from tiatoolbox.utils.transforms import imresize
-from tiatoolbox.wsicore.wsireader import WSIReader
+from tiatoolbox.annotation.storage import AnnotationStore
+from tiatoolbox.utils.transforms import imresize, locsize2bounds
+from tiatoolbox.utils.visualization import AnnotationRenderer, random_colors
+from tiatoolbox.wsicore.wsireader import WSIMeta, WSIReader
 
 defusedxml.defuse_stdlib()
 
@@ -53,7 +55,7 @@ class TilePyramidGenerator:
         tile_size: int = 256,
         downsample: int = 2,
         overlap: int = 0,
-    ):
+    ) -> None:
         self.wsi = wsi
         self.tile_size = tile_size
         self.overlap = overlap
@@ -101,7 +103,8 @@ class TilePyramidGenerator:
         )
 
     @property
-    def sub_tile_level_count(self):
+    def sub_tile_level_count(self) -> int:
+        """The number of sub-tile levels in the pyramid."""
         return 0
 
     @property
@@ -140,6 +143,7 @@ class TilePyramidGenerator:
         level: int,
         x: int,
         y: int,
+        res: int = 1,
         pad_mode: str = "constant",
         interpolation: str = "optimise",
     ) -> Image:
@@ -159,7 +163,7 @@ class TilePyramidGenerator:
             y (int):
                 The tile index in the y direction.
             pad_mode (str):
-                Method for padding when reading areas outside of the
+                Method for padding when reading areas outside the
                 input image. Default is constant (0 padding). This is
                 passed to `read_func` which defaults to
                 :func:`safe_padded_read`. See :func:`safe_padded_read`
@@ -171,7 +175,7 @@ class TilePyramidGenerator:
                 area, optimise. Linear most closely matches OpenSlide.
 
         Returns:
-            Image:
+            PIL.Image:
                 Pillow image of the tile.
 
         Example:
@@ -194,7 +198,7 @@ class TilePyramidGenerator:
         baseline_x = (x * self.tile_size * scale) - (self.overlap * scale)
         baseline_y = (y * self.tile_size * scale) - (self.overlap * scale)
         output_size = [self.output_tile_size] * 2
-        coord = [baseline_x, baseline_y]
+        coord = [int(baseline_x), int(baseline_y)]
         if level < self.sub_tile_level_count:
             output_size = self.output_tile_size // 2 ** (
                 self.sub_tile_level_count - level
@@ -210,15 +214,15 @@ class TilePyramidGenerator:
         # Don't print out any warnings about interpolation etc.
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            rgb = self.wsi.read_rect(
+            tile = self.wsi.read_rect(
                 coord,
-                size=output_size,
-                resolution=1 / scale,
+                size=[v * res for v in output_size],
+                resolution=res / scale,
                 units="baseline",
                 pad_mode=pad_mode,
                 interpolation=interpolation,
             )
-        return Image.fromarray(rgb)
+        return Image.fromarray(tile)
 
     def tile_path(self, level: int, x: int, y: int) -> Path:
         """Generate the path for a specified tile.
@@ -233,7 +237,7 @@ class TilePyramidGenerator:
                 The tile index in the y direction.
 
         Returns:
-            Path:
+            pathlib.Path:
                 A pathlib path object with two parts.
 
         """
@@ -264,7 +268,7 @@ class TilePyramidGenerator:
             ...   wsi=reader,
             ...   tile_size=256,
             ... )
-            >>>  tile_generator.dump(
+            >>> tile_generator.dump(
             ...    path="sample.gz.zip",
             ...    container="zip",
             ...    compression="gzip",
@@ -382,7 +386,7 @@ class ZoomifyGenerator(TilePyramidGenerator):
 
     """
 
-    def tile_group(self, level: int, x: int, y: int):
+    def tile_group(self, level: int, x: int, y: int) -> int:
         """Find the tile group for a tile index.
 
         Tile groups are numbered from level 0 (tile 0-0-0) and increment
@@ -397,6 +401,10 @@ class ZoomifyGenerator(TilePyramidGenerator):
             y (int):
                 The tile index in the y direction.
 
+        Raises:
+            IndexError:
+                If the level, x, y tile index is out of bounds.
+
         Returns:
             int:
                 The tile group for the specified tile.
@@ -405,9 +413,9 @@ class ZoomifyGenerator(TilePyramidGenerator):
         grid_size = np.array(self.tile_grid_size(level))
         if any(grid_size <= [x, y]):
             raise IndexError
-        cumsum = sum(np.prod(self.tile_grid_size(n)) for n in range(level))
+        cumulative_sum = sum(np.prod(self.tile_grid_size(n)) for n in range(level))
         index_in_level = np.ravel_multi_index((y, x), self.tile_grid_size(level)[::-1])
-        tile_index = cumsum + index_in_level
+        tile_index = cumulative_sum + index_in_level
         return tile_index // 256  # the tile group
 
     def tile_path(self, level: int, x: int, y: int) -> Path:
@@ -423,10 +431,184 @@ class ZoomifyGenerator(TilePyramidGenerator):
                 The tile index in the y direction.
 
         Returns:
-            Path:
+            pathlib.Path:
                 A pathlib path object with two parts.
 
         """
         g = self.tile_group(level, x, y)
         z = level
         return Path(f"TileGroup{g}") / f"{z}-{x}-{y}.jpg"
+
+
+class AnnotationTileGenerator(ZoomifyGenerator):
+    r"""Tile generator using an AnnotationRenderer to render tiles
+    showing annotations held in an AnnotationStore
+
+    Args:
+        info (WSIMeta):
+            An WSIMeta Object storing the metadata of the slide this
+            generator is rendering tiles for
+        Store (AnnotationStore):
+            An AnnotationStore Object containing annotations to be
+            rendered for given slide
+        renderer (AnnotationRenderer):
+            An AnnotationRenderer Object which will render annotations
+            belonging to a tile according to specified parameters
+        tile_size (int):
+            The size of tiles to generate. Default is 256. Note that the
+            output tile size will be :math:`\text{tile size} + 2
+            \times\text{overlap}`.
+        downsample (int):
+            The downsample factor between levels. Default is 2.
+        overlap (int):
+            The number of extra pixel to add to each edge of the tile.
+            Default is 0.
+
+    """
+
+    def __init__(
+        self,
+        info: WSIMeta,
+        store: AnnotationStore,
+        renderer: AnnotationRenderer = None,
+        tile_size: int = 256,
+        downsample: int = 2,
+        overlap: int = 0,
+    ):
+        super().__init__(None, tile_size, downsample, overlap)
+        self.info = info
+        self.store = store
+        if renderer is None:
+            renderer = AnnotationRenderer()
+        self.renderer = renderer
+        # if using blur, render overlapping tiles to minimise edge effects.
+        # factor of 1.5 below chosen empirically as a good balance between
+        # empirical visual quality and added rendering time.
+        self.overlap = int(1.5 * renderer.blur_radius)
+
+        output_size = [self.output_tile_size] * 2
+        self.empty_img = Image.fromarray(
+            np.zeros((output_size[0], output_size[1], 4), dtype=np.uint8)
+        )
+        if self.renderer.mapper == "categorical":
+            # get the possible categories for given score_prop from store
+            types = self.store.pquery(f"props['{self.renderer.score_prop}']")
+            # make a random dictionary colour map
+            colors = random_colors(len(types))
+            mapper = {key: (*color, 1) for key, color in zip(types, colors)}
+            self.renderer.mapper = lambda x: mapper[x]
+
+    def get_thumb_tile(self) -> Image:
+        """Return a thumbnail which fits the whole slide in one tile.
+
+        The thumbnail output size has the longest edge equal to the tile
+        size. The other edge preserves the original aspect ratio.
+
+        """
+        slide_dims = np.array(self.info.slide_dimensions)
+        scale = self.level_downsample(self.level_count - 1)
+        bounds = (0, 0, *slide_dims)
+        thumb = self.renderer.render_annotations(self.store, bounds, scale)
+        return Image.fromarray(thumb)
+
+    def level_dimensions(self, level: int) -> Tuple[int, int]:
+        """The total pixel dimensions of the tile pyramid at a given level.
+
+        Args:
+            level (int):
+                The level to calculate the dimensions for.
+
+        """
+        baseline_dims = self.info.slide_dimensions
+        level_dims = np.ceil(
+            np.divide(baseline_dims, self.level_downsample(level))
+        ).astype(int)
+        return tuple(level_dims)
+
+    @property
+    def level_count(self) -> int:
+        """Number of levels in the tile pyramid.
+
+        The number of levels is such that level_count - 1 is a 1:1 of
+        the slide baseline resolution (level 0 of the WSI).
+
+        """
+        wsi_to_tile_ratio = np.divide(self.info.slide_dimensions, self.tile_size)
+        # Levels where a tile contains only part of the wsi
+        super_level_count = np.ceil(np.log2(wsi_to_tile_ratio)).max()
+        total_level_count = super_level_count + 1 + self.sub_tile_level_count
+        return int(total_level_count)
+
+    def get_tile(
+        self,
+        level: int,
+        x: int,
+        y: int,
+        res: int = 1,
+        pad_mode: str = None,
+        interpolation: str = None,
+    ) -> Image:
+        """Render a tile at a given level and coordinate.
+
+        Note that levels are in the reverse order of those in WSIReader.
+        I.E. level 0 here corresponds to the lowest resolution whereas
+        level 0 in WSIReader corresponds to the maximum resolution
+        (baseline).
+
+        Args:
+            level (int):
+                The pyramid level of the tile starting from 0 (the whole
+                slide in one tile, 0-0-0).
+            x (int):
+                The tile index in the x direction.
+            y (int):
+                The tile index in the y direction.
+
+        Returns:
+            PIL.Image:
+                Pillow image of the tile.
+
+        Example:
+            >>> from tiatoolbox.tools.pyramid import AnnotationTileGenerator
+            >>> from tiatoolbox.wsicore.wsireader import WSIReader
+            >>> from tiatoolbox.annotation.storage import SQLiteStore
+            >>> wsi = WSIReader.open("sample.svs")
+            >>> SQ=SQLiteStore.from_geojson(geo_path)
+            >>> tile_generator = AnnotationTileGenerator(
+            ...   info=wsi.info,
+            ...   store=SQ,
+            ... )
+            >>> tile_0_0_0 = tile_generator.get_tile(level=0, x=0, y=0)
+
+        """
+        if pad_mode is not None or interpolation is not None:
+            warnings.warn(
+                "interpolation, pad_mode are unused by AnnotationTileGenerator"
+            )
+        if level < 0:
+            raise IndexError
+        if level > self.level_count:
+            raise IndexError("Invalid level.")
+
+        scale = self.level_downsample(level)
+        baseline_x = (x * self.tile_size * scale) - (self.overlap * scale)
+        baseline_y = (y * self.tile_size * scale) - (self.overlap * scale)
+        coord = [baseline_x, baseline_y]
+        if level < self.sub_tile_level_count:
+            output_size = self.output_tile_size // 2 ** (
+                self.sub_tile_level_count - level
+            )
+            output_size = np.repeat(output_size, 2).astype(int)
+            thumb = self.get_thumb_tile()
+            thumb.thumbnail(output_size)
+            return thumb
+        slide_dimensions = np.array(self.info.slide_dimensions)
+        if all(slide_dimensions < [baseline_x, baseline_y]):
+            raise IndexError
+
+        bounds = locsize2bounds(coord, [self.output_tile_size * scale] * 2)
+        tile = self.renderer.render_annotations(
+            self.store, bounds, scale, res, self.overlap
+        )
+
+        return Image.fromarray(tile)
